@@ -6,6 +6,8 @@ from fontTools.feaLib.builder import Builder, addOpenTypeFeatures, \
 from fontTools.feaLib.error import FeatureLibError
 from fontTools.ttLib import TTFont
 from fontTools.feaLib.parser import Parser
+from fontTools.feaLib import ast
+from fontTools.feaLib.lexer import Lexer
 import difflib
 import os
 import shutil
@@ -113,8 +115,8 @@ class BuilderTest(unittest.TestCase):
         expected = self.read_ttx(expected_ttx)
         if actual != expected:
             for line in difflib.unified_diff(
-                    expected, actual, fromfile=path, tofile=expected_ttx):
-                sys.stdout.write(line)
+                    expected, actual, fromfile=expected_ttx, tofile=path):
+                sys.stderr.write(line)
             self.fail("TTX output is different from expected")
 
     def build(self, featureFile):
@@ -131,17 +133,24 @@ class BuilderTest(unittest.TestCase):
             if tag in font:
                 font[tag].compile(font)
 
-    def check_fea2fea_file(self, name):
-        f = self.getpath("{}.fea".format(name))
-        p = Parser(f)
+    def check_fea2fea_file(self, name, base=None, parser=Parser):
+        fname = (name + ".fea") if '.' not in name else name
+        p = parser(self.getpath(fname))
         doc = p.parse()
-        tlines = self.normal_fea(doc.asFea().split("\n"))
-        with open(f, "r", encoding="utf-8") as ofile:
-            olines = self.normal_fea(ofile.readlines())
-        if olines != tlines:
-            for line in difflib.unified_diff(olines, tlines):
-                sys.stdout.write(line)
-            self.fail("Fea2Fea output is different from expected")
+        actual = self.normal_fea(doc.asFea().split("\n"))
+
+        with open(self.getpath(base or fname), "r", encoding="utf-8") as ofile:
+            expected = self.normal_fea(ofile.readlines())
+
+        if expected != actual:
+            fname = name.rsplit(".", 1)[0] + ".fea"
+            for line in difflib.unified_diff(
+                    expected, actual,
+                    fromfile=fname + " (expected)",
+                    tofile=fname + " (actual)"):
+                sys.stderr.write(line+"\n")
+            self.fail("Fea2Fea output is different from expected. "
+                      "Generated:\n{}\n".format("\n".join(actual)))
 
     def normal_fea(self, lines):
         output = []
@@ -361,6 +370,91 @@ class BuilderTest(unittest.TestCase):
             "Lookup blocks cannot be placed inside 'aalt' features",
             self.build, "feature aalt {lookup L {} L;} aalt;")
 
+    def test_extensions(self):
+        class ast_BaseClass(ast.MarkClass):
+            def asFea(self, indent=""):
+                return ""
+
+        class ast_BaseClassDefinition(ast.MarkClassDefinition):
+            def asFea(self, indent=""):
+                return ""
+
+        class ast_MarkBasePosStatement(ast.MarkBasePosStatement):
+            def asFea(self, indent=""):
+                if isinstance(self.base, ast.MarkClassName):
+                    res = ""
+                    for bcd in self.base.markClass.definitions:
+                        if res != "":
+                            res += "\n{}".format(indent)
+                        res += "pos base {} {}".format(bcd.glyphs.asFea(), bcd.anchor.asFea())
+                        for m in self.marks:
+                            res += " mark @{}".format(m.name)
+                        res += ";"
+                else:
+                    res = "pos base {}".format(self.base.asFea())
+                    for a, m in self.marks:
+                        res += " {} mark @{}".format(a.asFea(), m.name)
+                    res += ";"
+                return res
+
+        class testAst(object):
+            MarkBasePosStatement = ast_MarkBasePosStatement
+            def __getattr__(self, name):
+                return getattr(ast, name)
+
+        class testParser(Parser):
+            def parse_position_base_(self, enumerated, vertical):
+                location = self.cur_token_location_
+                self.expect_keyword_("base")
+                if enumerated:
+                    raise FeatureLibError(
+                        '"enumerate" is not allowed with '
+                        'mark-to-base attachment positioning',
+                        location)
+                base = self.parse_glyphclass_(accept_glyphname=True)
+                if self.next_token_ == "<":
+                    marks = self.parse_anchor_marks_()
+                else:
+                    marks = []
+                    while self.next_token_ == "mark":
+                        self.expect_keyword_("mark")
+                        m = self.expect_markClass_reference_()
+                        marks.append(m)
+                self.expect_symbol_(";")
+                return self.ast.MarkBasePosStatement(location, base, marks)
+
+            def parseBaseClass(self):
+                if not hasattr(self.doc_, 'baseClasses'):
+                    self.doc_.baseClasses = {}
+                location = self.cur_token_location_
+                glyphs = self.parse_glyphclass_(accept_glyphname=True)
+                anchor = self.parse_anchor_()
+                name = self.expect_class_name_()
+                self.expect_symbol_(";")
+                baseClass = self.doc_.baseClasses.get(name)
+                if baseClass is None:
+                    baseClass = ast_BaseClass(name)
+                    self.doc_.baseClasses[name] = baseClass
+                    self.glyphclasses_.define(name, baseClass)
+                bcdef = ast_BaseClassDefinition(location, baseClass, anchor, glyphs)
+                baseClass.addDefinition(bcdef)
+                return bcdef
+
+            extensions = {
+                'baseClass' : lambda s : s.parseBaseClass()
+            }
+            ast = testAst()
+
+        self.check_fea2fea_file(
+            "baseClass.feax", base="baseClass.fea", parser=testParser)
+
+    def test_markClass_same_glyph_redefined(self):
+        self.assertRaisesRegex(
+            FeatureLibError,
+            "Glyph acute already defined",
+            self.build,
+            "markClass [acute] <anchor 350 0> @TOP_MARKS;"*2)
+
 
 def generate_feature_file_test(name):
     return lambda self: self.check_feature_file(name)
@@ -381,4 +475,4 @@ for name in BuilderTest.TEST_FEATURE_FILES:
 
 
 if __name__ == "__main__":
-    unittest.main()
+    sys.exit(unittest.main())
