@@ -215,4 +215,5 @@ class HmtxTableTest(unittest.TestCase):
 
 
 if __name__ == "__main__":
-    unittest.main()
+    import sys
+    sys.exit(unittest.main())
