@@ -33,4 +33,5 @@ def _getpath(testfile):
 
 
 if __name__ == "__main__":
-    unittest.main()
+    import sys
+    sys.exit(unittest.main())
